# Mat 0.0, Nov 29, 2003
# Matrix operations
# V. Batagelj
#

mat.perm <- function(m,p)
{
  n <- length(p)
  mp <- matrix(nrow=n,ncol=n, 0 )
  rownames(mp) <- dimnames(m)[[1]][p]
  colnames(mp) <- rownames(mp)
  for (i in 1:n){
    for (j in 1:n) { mp[i,j] <- m[p[i],p[j]] }
  }
  mp
}

per.invert <- function(p)
{ q <- p
  n <- length(p)
  for (i in 1:n) { q[p[i]] <- i }
  q
}

vec.unit <- function(n)
{
  rep(1,n)
}

vec.sel <- function(n,p)
{
  q <- rep(0,n)
  for (i in 1:length(p)) { q[p[i]] <- 1 }
  q
}

mat.bin <- function(m)
{
  n <- nrow(m)
  mt <- matrix(nrow=n,as.integer(m>0))
  rownames(mt) <- rownames(m)
  colnames(mt) <- colnames(m)
  mt
}

mat.power <- function(m,k)
{
  n <- nrow(m)
  mt <- diag(n)
  rownames(mt) <- rownames(m)
  colnames(mt) <- colnames(m)
  if (k > 0) {
    i <- k
    ms <- m
    repeat
    {
      if ((i %% 2) == 1) { mt <- mt %*% ms }
      i <- i %/% 2
      if (i == 0)  break
      ms <- ms %*% ms
    }
  }
  mt
}

int.spw <- function(i,k,m)
{
  n <- nrow(m)
  if (i == 1) return(list(a=m+diag(n), b=m %*% m))
  if (i %% 2 == 0) {
    ab <- int.spw(i-1,k,m)
    list(a=ab$a + ab$b, b=if(i<k){ab$b %*% m}else{NULL})
  } else {
    ab <- int.spw(i %/% 2,k,m)
    p <- ab$b + diag(n)
    list(a=p %*% ab$a, b=if(i<k){ab$b %*% ab$b}else{NULL})
  }
}

mat.sumpow <- function(m,k)
{ n <- nrow(m)
  if (k == 0) return(diag(n))
  if (k == 1) return(diag(n)+m)
  int.spw(k,k,m)$a
}

mat.close.bin <- function(m)
{
  n <- nrow(m)
  mc <- m
  for (j in 1:n) { for (i in 1:n){
    if (mc[i,j] > 0) { mc[i,] <- as.integer((mc[i,] + mc[j,]) > 0) }
  }}
  mc
}

mat.dist <- function(m)
{
  n <- nrow(m); md <- m
  md[md==0] <- Inf
  diag(md) <- rep(0,n)
  for (k in 1:n) { for (i in 1:n){
    md[i,] <- pmin(md[i,],md[i,k] + md[k,])
  }}
  md
}

mat.paths <- function(m)
{
  n <- nrow(m); md <- m
  md[md==0] <- Inf
  diag(md) <- rep(0,n)
  mt <- md; mt[mt < Inf ] <- 0
  for (k in 1:n) { for (i in 1:n){ for (j in 1:n){
    d <- md[i,k] + md[k,j]
    if (md[i,j] > d) { md[i,j] <- d; mt[i,j] <- k }
  }}}
  list(dis=md,tra=mt)
}

int.path <- function(m,u,v)
{
  k <- m[u,v]
  if(k == 0) { return(NULL) }
  c(int.path(m,u,k),k,int.path(m,k,v))
}

vec.path <- function(m,u,v)
{
  if(m[u,v] == Inf) { return(NULL) }
  c(u,int.path(m,u,v),v)
}

vec.closeness <- function(m)
{
  n <- nrow(m)
  (n-1)/(mat.dist(m) %*% rep(1,n))
}

mat.geodesics <- function(m)
{
  n <- nrow(m)
  md <- m; md[m==0] <- Inf
  mc <- m; mc[m>0] <- 1
  for (k in 1:n) { for (u in 1:n){ for (v in 1:n){
    dst <- md[u,k] + md[k,v]
    if (md[u,v] >= dst) {
      cnt <- mc[u,k]*mc[k,v];
      if (md[u,v] == dst) {mc[u,v] <- mc[u,v] + cnt }
      else  {md[u,v] <- dst; mc[u,v] <- cnt }
    }
  }}}
  list(dis=md,cnt=mc)
}

vec.betweeness <- function(m){
  mt <- mat.geodesics(m);
  attach(mt)
  n <- nrow(m); bw <- rep(0,n)
  for (v in 1:n) {
    b <- 0
    for (u in 1:n) {for (w in 1:n) {
      if ((cnt[u,w] > 0) && (u != w) && (u != v) && (v != w) &&
            ((dis[u,v] + dis[v,w]) == dis[u,w]))
        {b <- b + cnt[u,v]*cnt[v,w] / cnt[u,w]}
    }}
    bw[v] <- b/((n-1)*(n-2))
  }
  bw
}

mat.prob <- function(m)
{
  d <- drop(m %*% rep(1,ncol(m)))
  (m + diag(d==0)) / (d + (d==0))
}

mat.rnd.ER <- function(n,p)
{
  m <- matrix(as.integer(runif(n*n)<p),nrow=n)
  diag(m) <- 0
  dimnames(m) <- list(paste('v',1:n,sep=''),paste('v',1:n,sep=''))
  m
}

mat.save.net <- function(fnet,m){
  net <- file(fnet,"w")
  n <- nrow(m)
  cat("*vertices",n,"\n",file=net)
  rn <- rownames(m)
  for (i in 1:n){
    cat(i," \"",rn[i],"\"\n",file=net,sep="")
  }
  cat("*matrix\n",file=net)
  for (i in 1:n){
    cat(m[i,],"\n",file=net)
  }
  close(net)
}

mat.absdet <- function(m){
  abs(prod(diag(qr(m)$qr)))
}

mat.det <- function(m){
  Re(prod(eigen(m,only.values=T)$values))
}

mat.trace <- function(m){
  sum(diag(m))
}

mat.row.sum <- function(m){
  m %*% rep(1,ncol(m))
}

mat.col.sum <- function(m){
  rep(1,nrow(m)) %*% m
}

mat.closeness <- function(m){
  (nrow(m)-1)/mat.row.sum(mat.dist(m))
}

# -------------------------------------------
